package de.hhu.ba.yoshikoWrapper.graphModel;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.cytoscape.model.CyEdge;
import org.cytoscape.model.CyNetwork;
import org.cytoscape.model.CyNode;
import org.cytoscape.model.subnetwork.CyRootNetwork;
import org.cytoscape.model.subnetwork.CySubNetwork;
import org.cytoscape.view.layout.CyLayoutAlgorithm;
import org.cytoscape.view.model.CyNetworkView;
import org.cytoscape.view.presentation.property.BasicVisualLexicon;
import org.cytoscape.view.vizmap.VisualStyle;
import org.cytoscape.view.vizmap.mappings.ContinuousMapping;
import de.hhu.ba.yoshikoWrapper.core.CyCore;
import de.hhu.ba.yoshikoWrapper.core.LocalizationManager;

public class YoshikoSolution {
	
	private CyNetwork originalGraph;
	
	public ArrayList<YoshikoCluster> cluster;
	public int id;
	
	public YoshikoSolution(CyNetwork originalGraph) {
		cluster = new ArrayList<YoshikoCluster>();
		this.originalGraph = originalGraph;
	}

	/**
	 * Transforms this solution in a meta-graph where each cluster is represented by one node
	 * The nodes are linked to subgraphs representing the cluster and also their size is scaled relative to cluster size
	 */
	public void exportMetaGraph() {
		
		CyNetwork metaGraph = CyCore.networkFactory.createNetwork();
		
		metaGraph.getRow(metaGraph).set(CyNetwork.NAME, LocalizationManager.get("metaGraph"));
		
		metaGraph.getDefaultNodeTable().createColumn("clusterSize", Integer.class, false);
		
		CyLayoutAlgorithm layout = CyCore.layoutAlgorithmManager.getDefaultLayout();

		
		//Map Cluster to Nodes for further reference
		HashMap<YoshikoCluster,CyNode> map = new HashMap<YoshikoCluster,CyNode>();
		
		//Convert original graph to root graph to allow embedding of nodes
		CyRootNetwork originalGraphAsRoot = CyCore.rootNetworkManager.getRootNetwork(originalGraph);
		
		//Add nodes
		for (YoshikoCluster c: cluster) {
			CyNode clusterNode = metaGraph.addNode();
			//Create nested graph and cluster subnet
			ArrayList<CyEdge> inducedEdges = new ArrayList<CyEdge>();
			for (CyNode n: c.nodes) {
				//Sadly Cytoscape doesnt provide a comfort function here
				List<CyEdge> adjacentEdges = originalGraph.getAdjacentEdgeList(n, CyEdge.Type.ANY);
				for (CyEdge e: adjacentEdges) {
					if (c.nodes.contains(e.getSource()) && c.nodes.contains(e.getTarget())) {
						inducedEdges.add(e);
					}
				}
			}
			CySubNetwork subnet = originalGraphAsRoot.addSubNetwork(c.nodes ,inducedEdges);

			subnet.getRow(subnet).set(CyNetwork.NAME, LocalizationManager.get("cluster")+" "+(c.id+1));
			
			CyCore.networkManager.addNetwork(subnet);
			//Create network view and register it
			CyNetworkView subnetView = CyCore.networkViewFactory.createNetworkView(subnet);
			CyCore.networkViewManager.addNetworkView(subnetView);
			//layout cluster
			CyCore.dialogTaskManager.execute(
				layout.createTaskIterator(
						subnetView, 
						layout.getDefaultLayoutContext(), 
						CyLayoutAlgorithm.ALL_NODE_VIEWS,
						null
				)
			);
			clusterNode.setNetworkPointer(subnet);
			//Set node attributes
			metaGraph.getRow(clusterNode).set("name", LocalizationManager.get("cluster")+" "+c.id);
			metaGraph.getRow(clusterNode).set("clusterSize",c.nodes.size());
			map.put(c, clusterNode);
		}
		
		//Add edges (O(|C|^2*|V|^2) can maybe be improved? //TODO
		for (YoshikoCluster c1:cluster) {
			for (YoshikoCluster c2:cluster) {
				if (
						metaGraph.containsEdge(map.get(c1),map.get(c2))
						|| c1 == c2
						) {
					continue;
				}
				clusterloop:
				for (CyNode c1n : c1.nodes) {
					for (CyNode c2n : c2.nodes) {
						if (originalGraph.containsEdge(c1n, c2n) || originalGraph.containsEdge(c2n, c1n)) {
							metaGraph.addEdge(map.get(c1), map.get(c2), false);
							break clusterloop;
						}
					}
				}
			}
		}
		
		CyNetworkView view = CyCore.networkViewFactory.createNetworkView(metaGraph);
		//layout solution
		CyCore.dialogTaskManager.execute(
			layout.createTaskIterator(
					view, 
					layout.getDefaultLayoutContext(), 
					CyLayoutAlgorithm.ALL_NODE_VIEWS,
					null
			)
		);
		
		CyCore.networkManager.addNetwork(metaGraph);
		CyCore.networkViewManager.addNetworkView(
				view
		);
		
		//define style for solution
		ContinuousMapping<Integer, Double> contMap = (ContinuousMapping<Integer, Double>)CyCore.continuousMappingFactory.createVisualMappingFunction(
				"clusterSize",
				Integer.class,
				BasicVisualLexicon.NODE_SIZE
			);
		VisualStyle vs= CyCore.visualStyleFactory.createVisualStyle("Meta-Graph Style");
		vs.addVisualMappingFunction(contMap);
		CyCore.visualMappingManager.addVisualStyle(vs);
		CyCore.cy.setCurrentNetworkView(view);
		//Apply visual style
		CyCore.visualMappingManager.setVisualStyle(vs, view);
		vs.apply(view);
		view.updateView();

	}
	
}
