package de.hhu.ba.yoshikoWrapper.gui;

import java.awt.Component;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;

import org.cytoscape.application.swing.CytoPanelComponent;
import org.cytoscape.application.swing.CytoPanelName;
import org.cytoscape.work.AbstractTask;
import org.cytoscape.work.TaskIterator;

import de.hhu.ba.yoshikoWrapper.core.AlgorithmTask;
import de.hhu.ba.yoshikoWrapper.core.CyCore;
import de.hhu.ba.yoshikoWrapper.core.GraphicsLoader;
import de.hhu.ba.yoshikoWrapper.core.LocalizationManager;
import de.hhu.ba.yoshikoWrapper.core.YoshikoLoader;

/**This class describes the Swing Panel that the user interacts with in cytoscape
 * @author Philipp Spohr, Aug 6, 2017
 *
 */
@SuppressWarnings("serial")
public class MainPanel extends ComfortPanel implements CytoPanelComponent {

	//SWING COMPONENTS
	
	private final YoshikoHeader header;
	
	/**
	 * Swing is a bit special sometimes, so a panel is used to wrap the lang-switcher.
	 * This prevents stretching of the component
	 */
	private final ComfortPanel langSwitcherWrapper;
	private final LanguageSwitcher langSwitcher;
	
	private final LibraryPanel libraryPanel;

	private final EditCostPanel ecPanel;
	
	
	private final ReductionRulesChooser reductionRulesChooser;

	private final OperationModePanel opModePanel;
	
	private final JCheckBox showAdvancedOptions;
	

	private final ArrayList<JComponent> advancedOptions;

	/**
	 * Main constructor, creates a new Panel and initializes subcomponents
	 */
	public MainPanel() {
				
		//SWING INIT
		this.setLayout(new FlowLayout());
		
		//Initialize Swing components
		header = new YoshikoHeader();
		
		langSwitcherWrapper = new ComfortPanel();
		langSwitcher = new LanguageSwitcher();
		langSwitcherWrapper.addAll(new JLabel(LocalizationManager.get("switchLanguage")),langSwitcher);
		langSwitcherWrapper.setLayout(new FlowLayout());
		
		libraryPanel = new LibraryPanel();
		
		ecPanel = new EditCostPanel();
		
		opModePanel = new OperationModePanel();
		
		reductionRulesChooser = new ReductionRulesChooser();
		
		JButton runButton = new JButton("RUN");
		runButton.addActionListener(buttonListener);

		
		showAdvancedOptions = new JCheckBox(LocalizationManager.get("showAdvanced"));
		showAdvancedOptions.addActionListener(toggleAdvancedOptionsListener);

		this.addAll(
				header,
				langSwitcherWrapper,
				showAdvancedOptions,
				libraryPanel,
				ecPanel,
				reductionRulesChooser,
				opModePanel,
				runButton
				);
		
		advancedOptions = new ArrayList<JComponent>();
		//Manage all advanced components separately to enable toggling
		advancedOptions.addAll(
				Arrays.asList(
						opModePanel,
						reductionRulesChooser
						)
		);
		
		showAdvancedOptions(false);

		this.setVisible(true);
	}
	
	private ActionListener toggleAdvancedOptionsListener = new ActionListener() {

		@Override
		public void actionPerformed(ActionEvent e) {
			showAdvancedOptions(showAdvancedOptions.isSelected());
		}
		
	};
	
	private void showAdvancedOptions(boolean show) {
		for (JComponent j : advancedOptions) {
			j.setVisible(show);
		}
	}

	/**
	 * ButtonListener for the "Run" Button
	 * Handles calling the algorithm and fetching/passing the arguments from swing components
	 */
	private ActionListener buttonListener = new ActionListener() {

		@Override
		public void actionPerformed(ActionEvent e) {
			if (YoshikoLoader.isLibraryLoaded()){
				AbstractTask yoshiko = new AlgorithmTask(
					CyCore.cy.getCurrentNetwork(),
					opModePanel.getTimeLimit(),
					ecPanel.getWeightColumn(),
					ecPanel.getPermanentColumn(),
					ecPanel.getForbiddenColumn(),
					ecPanel.getDefaultInsertionCost(),
					ecPanel.getDefaultDeletionCost(),
					reductionRulesChooser.getBitMask(),
					reductionRulesChooser.getMultFactor(),
					opModePanel.useTriangleCuts(),
					opModePanel.usePartitionCuts(), 
					opModePanel.useHeuristic(),
					opModePanel.getSolCount()
				);
				CyCore.dialogTaskManager.execute(new TaskIterator(1,yoshiko));
			}
			else {
				JOptionPane.showMessageDialog(	
					null,
					LocalizationManager.get("noLibTitle"),
					LocalizationManager.get("noLibMessage"),
					JOptionPane.ERROR_MESSAGE
				);
			}
		}
	};
	

	
	//GETTER / SETTER

	public Component getComponent() {
		return this;
	}

	/* (non-Javadoc)
	 * @see org.cytoscape.application.swing.CytoPanelComponent#getCytoPanelName()
	 */
	public CytoPanelName getCytoPanelName() {
		//By convention most plugins that provide a "toolbox"-like interface use the WEST orientation
		return CytoPanelName.WEST;
	}

	public String getTitle() {
		//TODO: Be creative I guess
		return "Yoshiko";
	}
	
	public ColumnMapper getColumnMapper() {
		return ecPanel.getColumnMapper();
	}
	
	public Icon getIcon() {
		return GraphicsLoader.getLogo();
	}


}
